package com.tarak.api.entity.Inventory.Items;

import java.util.Random;
import javax.swing.Icon;

public class Item
{
    //private CreativeTabs tabToDisplayOn = null;

    /** The RNG used by the Item subclasses. */
    protected static Random itemRand = new Random();



    /** The ID of this item. */
    public final int itemID;

    /** Maximum size of the stack. */
    protected int maxStackSize = 64;

    /** Maximum damage an item can handle. */
    private int maxDamage = 0;

    /** If true, render the object in full 3D, like weapons and tools. */
    protected boolean bFull3D = false;

    /**
     * Some items (like dyes) have multiple subtypes on same item, this is field define this behavior
     */
    protected boolean hasSubtypes = false;
    private Item containerItem = null;
    private String potionEffect = null;

    /** The unlocalized name of this item. */
    private String unlocalizedName;

    /** Icon index in the icons table. */
    protected Icon iconIndex;

    protected Item(int par1)
    {
        this.itemID = 256 + par1;


    }
    


    public Item setMaxStackSize(int par1)
    {
        this.maxStackSize = par1;
        return this;
    }

    /**
     * Returns 0 for /terrain.png, 1 for /gui/items.png
     */
    public int getSpriteNumber()
    {
        return 1;
    }

    /**
     * Gets an icon index based on an item's damage value
     */
    public Icon getIconFromDamage(int par1)
    {
        return this.iconIndex;
    }

    /**
     * Returns the icon index of the stack given as argument.
     */
    public final Icon getIconIndex(ItemStack par1ItemStack)
    {
        return this.getIconFromDamage(par1ItemStack.getItemDamage());
    }

//    /**
//     * Callback for item usage. If the item does something special on right clicking, he will have one of those. Return
//     * True if something happen and false if it don't. This is for ITEMS, not BLOCKS
//     */
//    public boolean onItemUse(ItemStack par1ItemStack, PlayerEntity par2PlayerEntity, World par3World, int par4, int par5, int par6, int par7, float par8, float par9, float par10)
//    {
//        return false;
//    }
//
//    /**
//     * Returns the strength of the stack against a given block. 1.0F base, (Quality+1)*2 if correct blocktype, 1.5F if
//     * sword
//     */
//    public float getStrVsBlock(ItemStack par1ItemStack, Block par2Block)
//    {
//        return 1.0F;
//    }

//    /**
//     * Called whenever this item is equipped and the right mouse button is pressed. Args: itemStack, world, PlayerEntity
//     */
//    public ItemStack onItemRightClick(ItemStack par1ItemStack, World par2World, PlayerEntity par3PlayerEntity)
//    {
//        return par1ItemStack;
//    }
//
//    public ItemStack onEaten(ItemStack par1ItemStack, World par2World, PlayerEntity par3PlayerEntity)
//    {
//        return par1ItemStack;
//    }

    /**
     * Returns the maximum size of the stack for a specific item. *Isn't this more a Set than a Get?*
     */
    public int getItemStackLimit()
    {
        return this.maxStackSize;
    }

    /**
     * Returns the metadata of the block which this Item (ItemBlock) can place
     */
    public int getMetadata(int par1)
    {
        return 0;
    }

    public boolean getHasSubtypes()
    {
        return this.hasSubtypes;
    }

    protected Item setHasSubtypes(boolean par1)
    {
        this.hasSubtypes = par1;
        return this;
    }

    /**
     * Returns the maximum damage an item can take.
     */
    public int getMaxDamage()
    {
        return this.maxDamage;
    }

    /**
     * set max damage of an Item
     */
    protected Item setMaxDamage(int par1)
    {
        this.maxDamage = par1;
        return this;
    }

    public boolean isDamageable()
    {
        return this.maxDamage > 0 && !this.hasSubtypes;
    }





//    /**
//     * Returns if the item (tool) can harvest results from the block type.
//     */
//    public boolean canHarvestBlock(Block par1Block)
//    {
//        return false;
//    }



    /**
     * Sets bFull3D to True and return the object.
     */
    public Item setFull3D()
    {
        this.bFull3D = true;
        return this;
    }

    /**
     * Returns True is the item is renderer in full 3D when hold.
     */
    public boolean isFull3D()
    {
        return this.bFull3D;
    }

    /**
     * Returns true if this item should be rotated by 180 degrees around the Y axis when being held in an entities
     * hands.
     */
    public boolean shouldRotateAroundWhenRendering()
    {
        return false;
    }

    /**
     * Sets the unlocalized name of this item to the string passed as the parameter, prefixed by "item."
     */
    public Item setUnlocalizedName(String par1Str)
    {
        this.unlocalizedName = par1Str;
        return this;
    }
//
//    /**
//     * Gets the localized name of the given item stack.
//     */
//    public String getLocalizedName(ItemStack par1ItemStack)
//    {
//        String var2 = this.getUnlocalizedName(par1ItemStack);
//        return var2 == null ? "" : StatCollector.translateToLocal(var2);
//    }

    /**
     * Returns the unlocalized name of this item.
     */
    public String getUnlocalizedName()
    {
        return "item." + this.unlocalizedName;
    }

    /**
     * Returns the unlocalized name of this item. This version accepts an ItemStack so different stacks can have
     * different names based on their damage or NBT.
     */
    public String getUnlocalizedName(ItemStack par1ItemStack)
    {
        return "item." + this.unlocalizedName;
    }

    public Item setContainerItem(Item par1Item)
    {
        this.containerItem = par1Item;
        return this;
    }

    /**
     * If this returns true, after a recipe involving this item is crafted the container item will be added to the
     * player's inventory instead of remaining in the crafting grid.
     */
    public boolean doesContainerItemLeaveCraftingGrid(ItemStack par1ItemStack)
    {
        return true;
    }

    /**
     * If this function returns true (or the item is damageable), the ItemStack's NBT tag will be sent to the client.
     */
    public boolean getShareTag()
    {
        return true;
    }

    public Item getContainerItem()
    {
        return this.containerItem;
    }

    /**
     * True if this Item has a container item (a.k.a. crafting result)
     */
    public boolean hasContainerItem()
    {
        return this.containerItem != null;
    }

//    public String getStatName()
//    {
//        return StatCollector.translateToLocal(this.getUnlocalizedName() + ".name");
//    }
//
//    public String func_77653_i(ItemStack par1ItemStack)
//    {
//        return StatCollector.translateToLocal(this.getUnlocalizedName(par1ItemStack) + ".name");
//    }
//
//    public int getColorFromItemStack(ItemStack par1ItemStack, int par2)
//    {
//        return 16777215;
//    }
//
//    /**
//     * Called each tick as long the item is on a player inventory. Uses by maps to check if is on a player hand and
//     * update it's contents.
//     */
//    public void onUpdate(ItemStack par1ItemStack, World par2World, Entity par3Entity, int par4, boolean par5) {}
//
//    /**
//     * Called when item is crafted/smelted. Used only by maps so far.
//     */
//    public void onCreated(ItemStack par1ItemStack, World par2World, PlayerEntity par3PlayerEntity) {}
//
//    /**
//     * false for all Items except sub-classes of ItemMapBase
//     */
//    public boolean isMap()
//    {
//        return false;
//    }
//
//    /**
//     * returns the action that specifies what animation to play when the items is being used
//     */
//    public EnumAction getItemUseAction(ItemStack par1ItemStack)
//    {
//        return EnumAction.none;
//    }

    /**
     * How long it takes to use or consume an item
     */
    public int getMaxItemUseDuration(ItemStack par1ItemStack)
    {
        return 0;
    }

    /**
     * called when the player releases the use item button. Args: itemstack, world, PlayerEntity, itemInUseCount
     */
    //public void onPlayerStoppedUsing(ItemStack par1ItemStack, World par2World, PlayerEntity par3PlayerEntity, int par4) {}

    /**
     * Sets the string representing this item's effect on a potion when used as an ingredient.
     */
    protected Item setPotionEffect(String par1Str)
    {
        this.potionEffect = par1Str;
        return this;
    }

    /**
     * Returns a string representing what this item does to a potion.
     */
    public String getPotionEffect()
    {
        return this.potionEffect;
    }

    /**
     * Returns true if this item serves as a potion ingredient (its ingredient information is not null).
     */
    public boolean isPotionIngredient()
    {
        return this.potionEffect != null;
    }



    public boolean hasEffect(ItemStack par1ItemStack)
    {
        return par1ItemStack.isItemEnchanted();
    }

//    /**
//     * Return an item rarity from EnumRarity
//     */
//    public EnumRarity getRarity(ItemStack par1ItemStack)
//    {
//        return par1ItemStack.isItemEnchanted() ? EnumRarity.rare : EnumRarity.common;
//    }

    /**
     * Checks isDamagable and if it cannot be stacked
     */
    public boolean isItemTool(ItemStack par1ItemStack)
    {
        return this.getItemStackLimit() == 1 && this.isDamageable();
    }

//    protected MovingObjectPosition getMovingObjectPositionFromPlayer(World par1World, PlayerEntity par2PlayerEntity, boolean par3)
//    {
//        float var4 = 1.0F;
//        float var5 = par2PlayerEntity.prevRotationPitch + (par2PlayerEntity.rotationPitch - par2PlayerEntity.prevRotationPitch) * var4;
//        float var6 = par2PlayerEntity.prevRotationYaw + (par2PlayerEntity.rotationYaw - par2PlayerEntity.prevRotationYaw) * var4;
//        double var7 = par2PlayerEntity.prevPosX + (par2PlayerEntity.posX - par2PlayerEntity.prevPosX) * (double)var4;
//        double var9 = par2PlayerEntity.prevPosY + (par2PlayerEntity.posY - par2PlayerEntity.prevPosY) * (double)var4 + 1.62D - (double)par2PlayerEntity.yOffset;
//        double var11 = par2PlayerEntity.prevPosZ + (par2PlayerEntity.posZ - par2PlayerEntity.prevPosZ) * (double)var4;
//        Vec3 var13 = par1World.getWorldVec3Pool().getVecFromPool(var7, var9, var11);
//        float var14 = MathHelper.cos(-var6 * 0.017453292F - (float)Math.PI);
//        float var15 = MathHelper.sin(-var6 * 0.017453292F - (float)Math.PI);
//        float var16 = -MathHelper.cos(-var5 * 0.017453292F);
//        float var17 = MathHelper.sin(-var5 * 0.017453292F);
//        float var18 = var15 * var16;
//        float var20 = var14 * var16;
//        double var21 = 5.0D;
//        Vec3 var23 = var13.addVector((double)var18 * var21, (double)var17 * var21, (double)var20 * var21);
//        return par1World.rayTraceBlocks_do_do(var13, var23, par3, !par3);
//    }

    /**
     * Return the enchantability factor of the item, most of the time is based on material.
     */
    public int getItemEnchantability()
    {
        return 0;
    }

    public boolean requiresMultipleRenderPasses()
    {
        return false;
    }

    /**
     * Gets an icon index based on an item's damage value and the given render pass
     */
    public Icon getIconFromDamageForRenderPass(int par1, int par2)
    {
        return this.getIconFromDamage(par1);
    }


}